package com.googlecode.mad.mvntools.settingsutil.utils;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * this Util is parsing the pom.properties file and gets provides the buildnumber and builddate <br>
 * <br>
 * LastChangedBy: $Author: marthaler.worb@gmail.com $<br>
 * LastChangedDate: $Date: 2010-12-21 20:39:28 +0100 (Tue, 21 Dec 2010) $<br>
 * 
 * @author (daniel marthaler)
 * @version $Revision: 17 $
 * @since 24.12.2010
 */
public final class MavenVersionUtil {
    
    private static final String     VERSION       = "version=";
    
    private static MavenVersionUtil instance      = null;
    
    private Date                    compileadet   = new Date(0L);
    
    private String                  dateString    = "";
    
    private String                  version       = "?";
    
    private final String            sourcepattern = "#EEE MMM dd HH:mm:ss z yyyy";
    
    private final String            targetpattern = "dd.MM.yy HH:mm";
    
    private final String            file          = "META-INF/maven/[group]/[artid]/pom.properties";
    
    private MavenVersionUtil() {
        
    }
    
    public static synchronized MavenVersionUtil getInstance() {
        if (MavenVersionUtil.instance == null) {
            MavenVersionUtil.instance = new MavenVersionUtil();
        }
        return MavenVersionUtil.instance;
    }
    
    /**
     * @return the date-string
     */
    public String getDateString(final String groupid, final String artifactid) {
        this.init(this.file.replace("[group]", groupid).replace("[artid]", artifactid));
        if ("".equals(this.dateString)) {
            final SimpleDateFormat fm = new SimpleDateFormat(this.targetpattern);
            this.dateString = fm.format(this.compileadet);
        }
        return this.dateString;
    }
    
    /**
     * @return the actual version provided in the pom.xml
     */
    
    public String getVersionString(final String groupid, final String artifactid) {
        this.init(this.file.replace("[group]", groupid).replace("[artid]", artifactid));
        return this.version;
    }
    
    private void init(final String file) {
        final URL url = MavenVersionUtil.class.getClassLoader().getResource(file);
        
        if (url == null) {
            
            return;
        }
        
        InputStream oin = null;
        try {
            oin = new BufferedInputStream(url.openStream());
        } catch (final IOException e1) {
            
        }
        
        String line;
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new InputStreamReader(oin, "UTF-8"));
        } catch (final UnsupportedEncodingException e) {
            
        }
        try {
            while ((line = reader.readLine()) != null) {
                
                if (line.contains("CEST") || line.contains("CET")) {
                    final SimpleDateFormat format = new SimpleDateFormat(this.sourcepattern, Locale.US);
                    try {
                        this.compileadet = format.parse(line);
                    } catch (final ParseException e) {
                        
                    }
                    
                }
                
                if (line.contains("version")) {
                    this.version = line.replace(MavenVersionUtil.VERSION, "");
                }
            }
        } catch (final IOException e) {
            
        }
    }
}
